# load libraries
library(TCGA2STAT)
library(biomaRt)
library(KEGG.db)

# load data
GEdata <- getTCGA(disease="LUSC", data.type="RNASeq", 
                  type="RPKM", clinical=TRUE, 
                  cvars="pathologicstage")
CNdata <- getTCGA(disease="LUSC", data.type="CNA_CGH", 
                  clinical=TRUE, cvars="pathologicstage")

# determine overlap in samples and features 
sampleID   <- intersect(substr(colnames(CNdata$dat), 1, 12), 
                        substr(colnames(GEdata$dat), 1, 12))
featureID  <- intersect(rownames(CNdata$dat), rownames(GEdata$dat))

# match data to intersecting samples and features 
stages <- GEdata$clinical[match(sampleID, rownames(GEdata$clinical)), 7]
GEdata <- GEdata$dat[match(featureID, rownames(GEdata$dat)), 
                     match(sampleID, substr(colnames(GEdata$dat), 1, 12))]
CNdata <- CNdata$dat[match(featureID, rownames(CNdata$dat)), 
                     match(sampleID, substr(colnames(CNdata$dat), 1, 12))]

# dichotomize stages
stages[which(substr(stages, 7, 8) == "ii" | substr(stages, 7, 8) == "iv")] <- 1
stages[which(substr(stages, 7, 8) == "ia" | substr(stages, 7, 8) == "ib")] <- 0

# remove sample with missing stage information
idRem  <- which(is.na(stages))
stages <- stages[-idRem]
GEdata <- GEdata[,-idRem]
CNdata <- CNdata[,-idRem]

# KEGG pathway list with entrezIDs
xx <- as.list(KEGGPATHID2EXTID)

# get gene info via biomaRt
ensembl <- useMart("ENSEMBL_MART_ENSEMBL", host="www.ensembl.org", 
                   dataset="hsapiens_gene_ensembl")
gInfo   <- getBM(attributes=c("entrezgene", "hgnc_symbol"), mart=ensembl)

# limit omics data to that of toll-like receptor pathway
gInfo   <- gInfo[match(xx[[17]], gInfo[,1]),]
idSel   <- match(gInfo[,2], rownames(GEdata))
idSel   <- idSel[!is.na(idSel)]
CNdata  <- CNdata[idSel,]
GEdata  <- GEdata[idSel,]

# remove genes with more than 50% zeros.
# removes only genes from the ... family, which are hardly, if ever expressed
idRem  <- which(apply(GEdata, 1, function(Z){ sum(Z==0) }) / ncol(GEdata) > 0.5)
CNdata <- CNdata[-idRem,]
GEdata <- GEdata[-idRem,]

# separate data per stage and type
GE0    <- GEdata[,stages==0]
GE1    <- GEdata[,stages==1]
CN0    <- CNdata[,stages==0]
CN1    <- CNdata[,stages==1]

# gaussianize data (cf. Liu et al, JMLR)
GE0 <- apply(GE0, 2, function(Z){ qnorm(ecdf(c(Z, max(Z)+1))(Z)) })
GE1 <- apply(GE1, 2, function(Z){ qnorm(ecdf(c(Z, max(Z)+1))(Z)) })
CN0 <- apply(CN0, 2, function(Z){ qnorm(ecdf(c(Z, max(Z)+1))(Z)) })
CN1 <- apply(CN1, 2, function(Z){ qnorm(ecdf(c(Z, max(Z)+1))(Z)) })

# save data
save(CN0, CN1, GE0, GE1, CNdata, GEdata, stages, file="lungTCGAdata.Rdata") 

