# load functions
library(Rcpp)
sourceCpp("porridge.cpp")

# load data
load("lungTCGAdata.Rdata") 

# analysis of stage 0, for target construction
Y0       <- cbind(t(GE0), t(CN0))
molGroup <- c(rep(0, nrow(GE0)), rep(1, nrow(CN0)))

# define parameters 
p          <- ncol(Y0)/2
adjMat     <- rbind(cbind(matrix(1, p, p), diag(p)), cbind(diag(p), diag(p))) 
zeros      <- which(adjMat==0, arr.ind=TRUE) 

# make k-folds as list
fold         <- 5
fold         <- max(min(ceiling(fold), nrow(Y0)), 2)
fold         <- rep(1:fold, ceiling(nrow(Y0)/fold))[1:nrow(Y0)]
shuffle      <- sample(1:nrow(Y0), nrow(Y0))
folds        <- split(shuffle, as.factor(fold))
names(folds) <- NULL

# assess CV-loss (takes a while)
CVs <- matrix(ncol=3, nrow=0)
for (u in 1:100){
    GElambda <- exp(runif(1, -13, 0))
    CNlambda <- exp(runif(1, -13, 0))
    CVloss   <- .armaKCVlossR_groups(
                       c(GElambda, CNlambda),
                       Y=Y0, 
                       fold=folds,
                       groups=molGroup,
                       target=matrix(0, 2*p, 2*p), 
                       penalize_diag=TRUE,
                       zerosR=zeros[,1]-1, 
                       zerosC=zeros[,2]-1, 
                       nInit=100, 
                       minSuccDiff=0.00001)
    CVs      <- rbind(CVs, c(GElambda, CNlambda, CVloss))
}

# smooth the cv loglikelihood
ll1    <- log(CVs[,1])  
ll2    <- log(CVs[,2])  
ll12   <- log(CVs[,1])     * log(CVs[,2])  
ll11   <- (log(CVs[,1]))^2 
ll22   <- (log(CVs[,2]))^2 
ll111  <- (log(CVs[,1]))^3 
ll222  <- (log(CVs[,2]))^3
ll1111 <- (log(CVs[,1]))^4 
ll2222 <- (log(CVs[,2]))^4
ll112  <- (log(CVs[,1]))^2 * log(CVs[,2])  
ll122  <- log(CVs[,1])     * (log(CVs[,2]))^2  
ll1112 <- (log(CVs[,1]))^3 * log(CVs[,2])  
ll1222 <- log(CVs[,1])     * (log(CVs[,2]))^3
ll1122 <- (log(CVs[,1]))^2 * (log(CVs[,2]))^2
lmFit  <- lm(CVs[,3] ~ ll1   + ll2   + ll12   + ll11   + ll22   + ll111  + ll222 + 
                       ll112 + ll122 + ll1111 + ll2222 + ll1122 + ll1112 + ll1222)

# create a sequence of incrementally increasing values
xgrid <-  seq(min(ll1), max(ll1), 0.11)
ygrid <-  seq(min(ll2), max(ll2), 0.1)

# Generate a dataframe with every possible combination of wt and hp
xygrid <-  expand.grid(ll1= xgrid, ll1=ygrid)
xygrid <- cbind(xygrid,         
                xygrid[,1] * xygrid[,2], 
                (xygrid[,1])^2, (xygrid[,2])^2,
                (xygrid[,1])^3, (xygrid[,2])^3,  
                (xygrid[,1])^2 * xygrid[,2], 
                xygrid[,1] * (xygrid[,2])^2,
                (xygrid[,1])^4, (xygrid[,2])^4,  
                (xygrid[,1])^2 * (xygrid[,2])^2, 
                (xygrid[,1]^3) * xygrid[,2], 
                (xygrid[,2]^3) * xygrid[,1])
colnames(xygrid) <- c("ll1", "ll2", "ll12", "ll11", "ll22",  "ll111", "ll222", 
                      "ll112", "ll122", "ll1111", "ll2222", "ll1122", "ll1112", "ll1222")

# feed the dataframe into the model
gridFit <-  matrix(predict(lmFit, newdata=xygrid), nrow=length(xgrid), 
                                                   ncol=length(ygrid), byrow=FALSE)

# extract optimal lambdas
optLambdas <- c(exp(xgrid[which.min(apply(gridFit, 1, min))]), 
                exp(ygrid[which.min(apply(gridFit, 2, min))]))

# plot contour
contour(xgrid, ygrid, -gridFit, levels=c(66, 65, 60, 50, 30, 0, -50, -125, -200), 
	xlab=expression(paste("log(", lambda[ge], ")", sep="")), 
	ylab=expression(paste("log(", lambda[cn], ")", sep="")), 
        main=expression(paste("5-fold CV loglikelihood, cancer, ", T, "=", 0[pp], sep="")))
points(log(optLambdas[1]), log(optLambdas[2]), pch=20, col="red")

# construct optimal penalty matrix
Y0                  <- cbind(t(GE0), t(CN0))
p                   <- ncol(Y0)/2
adjMat              <- rbind(cbind(matrix(1, p, p), diag(p)), cbind(diag(p), diag(p))) 
zeros               <- which(adjMat==0, arr.ind=TRUE) 
lambdaMat           <- matrix(mean(optLambdas), 2*p, 2*p)
lambdaMat[1:p, 1:p] <- optLambdas[1] 
diag(lambdaMat)     <- 10^(-10)
lambdaMat[zeros]    <- 10^(10) 

# construct target for grp1 from grp0
S       <- t(Y0) %*% Y0 / nrow(Y0)
Pcnt    <- .armaRidgePgen(S, lambdaMat,target=matrix(0,2*p,2*p), 
                          nInit=100, minSuccDiff=0.00001)
save(Pcnt, file="ridgePcancer.Rdata") 






